@@ -3,7 +3,7 @@
 #include "hashtable.h"
 #ifdef HAVE_AL
 mp3Loader *alutLoadMP3p = 0;
-Hashtable<std::string, ALuint,char [127]> soundHash;
+Hashtable<std::string, ALuint, 127> soundHash;
 unsigned int maxallowedsingle=10;
 unsigned int maxallowedtotal=40;
 float scalepos;
