@@ -1,5 +1,8 @@
 #ifndef AL_GLOBALS_H_
 #define AL_GLOBALS_H_
+
+/* #undef SOUND_DEBUG */
+
 #ifdef HAVE_AL
 #ifdef __APPLE__
 #include <al.h>
@@ -19,12 +22,12 @@ struct OurSound{
 };
 extern float scalepos;
 extern float scalevel;
-template <class a,class b,class c> class Hashtable;
+template <typename a,typename b,int c> class Hashtable;
 
 extern std::vector <ALuint> unusedsrcs;
 extern std::vector<ALuint> buffers;
 extern std::vector <OurSound> sounds;
-extern Hashtable<std::string, ALuint,char [127]> soundHash;
+extern Hashtable<std::string, ALuint,127> soundHash;
 typedef ALboolean (mp3Loader)(ALuint, ALvoid *, ALint);
 extern mp3Loader *alutLoadMP3p;
 extern unsigned int maxallowedsingle;
