@@ -1,4 +1,7 @@
-#ifdef USE_BOOST_129
+#ifdef USE_BOOST_131
+#if defined (_MSC_VER) && _MSC_VER<=1200
+#define Vector Vactor
+#endif
 #include <boost/python/tuple.hpp>
 
 namespace boost { namespace python { namespace detail {
@@ -19,4 +22,7 @@ tuple_base::tuple_base(object_cref sequence)
 {}
 
 }}}  // namespace boost::python
+#if defined (_MSC_VER) && _MSC_VER<=1200
+#undef Vector
 #endif
+#endif
\ No newline at end of file
