@@ -12,7 +12,7 @@ class MyIndHalo {
   MyIndHalo (const QVector &loc, const Vector &size);
 };
 void DoParticles (QVector pos, float hullpercent, const Vector &velocity, float radial_size, float hullpct,int faction);
-void LaunchOneParticle (const Matrix & mat, const Vector & vel,unsigned int seed, Mesh * mesh, float hullpct,int faction);
+void LaunchOneParticle (const Matrix & mat, const Vector & vel,unsigned int seed, Unit * mesh, float hullpct,int faction);
 class HaloSystem {
  private:
   std::vector <MyIndHalo> halo;
@@ -33,7 +33,7 @@ class HaloSystem {
   }
   unsigned int  AddHalo (const char * filename, const QVector &loc, const Vector &size, const GFXColor & col, std::string halo_type/*when it grows*/, float activation_speed);
   bool ShouldDraw (float speedsquared);
-  void Draw (const Matrix & trans, const Vector & scale, short halo_alpha, float nebdist, float hullpercentage, const Vector & velocity, int faction);//draws smoke and damage fx
+  void Draw (const Matrix & trans, const Vector & scale, int halo_alpha, float nebdist, float hullpercentage, const Vector & velocity, int faction);//draws smoke and damage fx
   //  unsigned int size(); {return halo.size();}
   void SetPosition (unsigned int which, const QVector &loc);
   void SetSize (unsigned int which, const Vector &scale);
