@@ -18,6 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
+#define GL_EXT_texture_env_combine 1
 #include "gl_globals.h"
 #include "vs_globals.h"
 #include "vegastrike.h"
@@ -98,11 +99,11 @@ GFXBOOL /*GFXDRVAPI*/ GFXCreateTexture(int width, int height, TEXTUREFORMAT text
 {
   int dummy=0;
   if (!isPowerOfTwo (width,dummy)) {
-    fprintf (stderr,"Width %d not a power of two",width);
+    VSFileSystem::vs_fprintf (stderr,"Width %d not a power of two",width);
     //    assert (false);
   }
   if (!isPowerOfTwo (height,dummy)) {
-    fprintf (stderr,"Height %d not a power of two",height);
+    VSFileSystem::vs_fprintf (stderr,"Height %d not a power of two",height);
     //    assert (false);
     
   }
@@ -111,14 +112,14 @@ GFXBOOL /*GFXDRVAPI*/ GFXCreateTexture(int width, int height, TEXTUREFORMAT text
   *handle = 0;
   while (*handle<textures.size()) {
     if (!textures[*handle].alive){
-      fprintf (stderr,"got dead tex");
+      //VSFileSystem::vs_fprintf (stderr,"got dead tex");
       break;
     }else{
       (*handle)++;
     }
   }
   if ((*handle)==textures.size()) {
-    fprintf (stderr,"!");
+    VSFileSystem::vs_fprintf (stderr,"x");
     textures.push_back(GLTexture());
     textures.back().palette=NULL;
     textures.back().alive=GFXTRUE;
@@ -133,16 +134,16 @@ GFXBOOL /*GFXDRVAPI*/ GFXCreateTexture(int width, int height, TEXTUREFORMAT text
     break;
   case CUBEMAP: textures [*handle].targets=GL_TEXTURE_CUBE_MAP_EXT;
     WrapMode = GL_CLAMP;
-    fprintf (stderr, "stage %d, wid %d, hei %d",texturestage,width,height);
+    VSFileSystem::vs_fprintf (stderr, "stage %d, wid %d, hei %d",texturestage,width,height);
     break;
   }
-  fprintf (stderr,"!");  
+  VSFileSystem::vs_fprintf (stderr,"y");  
   textures[*handle].name = *handle+1; //for those libs with stubbed out handle gen't
-  //fprintf (stderr,"Texture Handle %d",*handle);
+  //VSFileSystem::vs_fprintf (stderr,"Texture Handle %d",*handle);
   textures[*handle].alive = GFXTRUE;
   textures[*handle].texturestage = texturestage;
   textures[*handle].mipmapped = mipmap;
-  fprintf (stderr,"!");
+  VSFileSystem::vs_fprintf (stderr,"z");
   glGenTextures (1,&textures[*handle].name);
   glBindTexture (textures[*handle].targets,textures[*handle].name);
   activetexture[texturestage]=*handle;
@@ -169,12 +170,12 @@ GFXBOOL /*GFXDRVAPI*/ GFXCreateTexture(int width, int height, TEXTUREFORMAT text
   textures[*handle].height = height;
   textures[*handle].palette=NULL;
   if (palette&&textureformat == PALETTE8){
-    fprintf (stderr," palette ");  
+    VSFileSystem::vs_fprintf (stderr," palette ");  
     textures[*handle].palette = (GLubyte *)malloc (sizeof (GLubyte)*1024);
       ConvertPalette(textures[*handle].palette, (unsigned char *)palette);
     }
   textures[*handle].textureformat = GetUncompressedTextureFormat(textureformat);
-  fprintf (stderr,"!");
+  VSFileSystem::vs_fprintf (stderr,"w");
   //  GFXActiveTexture(0);
   return GFXTRUE;
 }
@@ -324,7 +325,6 @@ GFXBOOL /*GFXDRVAPI*/ GFXTransferTexture (unsigned char *buffer, int handle,  TE
   }
   int error;
   unsigned char * tempbuf = NULL;
-  unsigned char * tbuf =NULL;
   GLenum internalformat;
   GLenum image2D=GetImageTarget (imagetarget);
   glBindTexture(textures[handle].targets, textures[handle].name);
@@ -351,22 +351,27 @@ GFXBOOL /*GFXDRVAPI*/ GFXTransferTexture (unsigned char *buffer, int handle,  TE
 		static int fullout = XMLSupport::parse_int(vs_config->getVariable("graphics","detail_texture_full_color","1"))-1;
 		float numdivisors = logsize>fullout+blankout?(1./(logsize-fullout-blankout)):1;
 		float detailscale=1;
+                glTexImage2D(image2D,count,internalformat,width,height,0,textures[handle].textureformat,GL_UNSIGNED_BYTE,buffer);
+                
 		while(1){
-			glTexImage2D(image2D,count,internalformat,width,height,0,textures[handle].textureformat,GL_UNSIGNED_BYTE,buffer);
 			if (width==1&&height==1)
 				break;
 			int newwidth = width>2?width/2:1;
 			int newheight = height>2?height/2:1;
-			tempbuf=NULL;
-			count++;			
-			DownSampleTexture(&tempbuf,buffer,height,width,(internformat==PALETTE8?1:(internformat==RGBA32?4:3))* sizeof(unsigned char ),handle,newheight,newwidth,detail_texture?detailscale:1);
+			count++;
+			unsigned char * tbuf = NULL;
+			DownSampleTexture(&tbuf,tempbuf?tempbuf:buffer,height,width,(internformat==PALETTE8?1:(internformat==RGBA32?4:3))* sizeof(unsigned char ),handle,newheight,newwidth,detail_texture?detailscale:1);
+                        if (tempbuf) free(tempbuf);
+                        tempbuf = tbuf;
 			if (count>fullout) {
 				detailscale-=numdivisors;
 			}
 			if (detailscale<0)
 				detailscale=0;
-			buffer=tempbuf;
+			glTexImage2D(image2D,count,internalformat,width,height,0,textures[handle].textureformat,GL_UNSIGNED_BYTE,tempbuf);
 		}
+                if (tempbuf) free(tempbuf);
+                tempbuf=NULL;
     }else {
       glTexImage2D(image2D, 0, internalformat, textures[handle].width, textures[handle].height, 0, textures[handle].textureformat, GL_UNSIGNED_BYTE, buffer);
 	}
@@ -398,7 +403,7 @@ GFXBOOL /*GFXDRVAPI*/ GFXTransferTexture (unsigned char *buffer, int handle,  TE
 #endif
     {
       int nsize = 4*textures[handle].height*textures[handle].width;
-      tbuf =(unsigned char *) malloc (sizeof(unsigned char)*nsize);
+      unsigned char * tbuf =(unsigned char *) malloc (sizeof(unsigned char)*nsize);
       //      textures[handle].texture = tbuf;
       int j =0;
       for (int i=0; i< nsize; i+=4)
@@ -427,6 +432,11 @@ GFXBOOL /*GFXDRVAPI*/ GFXTransferTexture (unsigned char *buffer, int handle,  TE
 void /*GFXDRVAPI*/ GFXDeleteTexture (int handle) {
   if (textures[handle].alive) {
     glDeleteTextures(1, &textures[handle].name);
+    for (size_t i=0;i<sizeof(activetexture)/sizeof(int);++i) {
+      if (activetexture[i]==handle) {
+        activetexture[i]=-1;
+      }
+    }
   }
   if (textures[handle].palette) {
     free  (textures[handle].palette);
@@ -535,15 +545,15 @@ void GFXTextureEnv (int stage, GFXTEXTUREENVMODES mode) {
 		glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,type);
 		break;
 	case GFXINTERPOLATETEXTURE:
-		type = GL_INTERPOLATE_EXT;
+		type = GL_INTERPOLATE_ARB;
 		goto COMBINERS;
 	case GFXADDSIGNEDTEXTURE:
-		type = GL_ADD_SIGNED_EXT;		
+		type = GL_ADD_SIGNED_ARB;		
 	  COMBINERS:
-		glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_COMBINE_EXT);
-		glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE0_RGB_EXT,GL_PREVIOUS_EXT);
-		glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE1_RGB_EXT,GL_TEXTURE);
-		glTexEnvi(GL_TEXTURE_ENV,GL_COMBINE_RGB_EXT,type);
+		glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_COMBINE_ARB);
+		glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE0_RGB_ARB,GL_PREVIOUS_ARB);
+		glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE1_RGB_ARB,GL_TEXTURE);
+		glTexEnvi(GL_TEXTURE_ENV,GL_COMBINE_RGB_ARB,type);
 		break;
 	}
 }
