@@ -1,8 +1,7 @@
-#ifdef NETCOMM
-
 #include "networking/networkcomm.h"
 
 NetworkCommunication::NetworkCommunication() {}
+NetworkCommunication::NetworkCommunication( float minfreq, float maxfreq, bool video, bool secured, string method) {}
 NetworkCommunication::~NetworkCommunication() {}
 
 void	NetworkCommunication::AddToSession( ClientPtr clt) {}
@@ -11,12 +10,15 @@ int		NetworkCommunication::InitSession( float freq) { return 0;}
 int		NetworkCommunication::DestroySession() { return 0;}
 
 //void	NetworkCommunication::SendImage( SOCKETALT & socket) {}
+void	NetworkCommunication::StartWebcamTransfer() {}
+void	NetworkCommunication::StopWebcamTransfer() {}
 void	NetworkCommunication::SendSound( SOCKETALT & socket, ObjSerial serial) {}
 void	NetworkCommunication::SendMessage( SOCKETALT & socket, ObjSerial serial, string message) {}
-void	NetworkCommunication::RecvSound( char * sndbuffer, int length, bool encrypted) {}
+void	NetworkCommunication::RecvSound( const char * sndbuffer, int length, bool encrypted) {}
 void	NetworkCommunication::RecvMessage( string message, bool encrypted) {}
 
 char *	NetworkCommunication::GetWebcamCapture() { return NULL;}
+char *	NetworkCommunication::GetWebcamFromNetwork(int & length) { return NULL; }
 char	NetworkCommunication::HasWebcam() { return 0;}
 char	NetworkCommunication::HasPortaudio() { return 0;}
 void	NetworkCommunication::SwitchWebcam() {}
@@ -26,5 +28,3 @@ void	NetworkCommunication::GenerateKey() {}
 string	NetworkCommunication::EncryptBuffer( const char * buffer, unsigned int length) { return string("");}
 string	NetworkCommunication::DecryptBuffer( const char * buffer, unsigned int length) { return string("");}
 
-#endif
-
