@@ -11,6 +11,9 @@
 #include "vs_globals.h"
 #include "endianness.h"
 #include <assert.h>
+#include "networking/netserver.h"
+
+extern StarSystem * GetLoadedStarSystem( const char * system);
 
 ZoneMgr::ZoneMgr()
 {
@@ -24,25 +27,29 @@ void			ZoneMgr::addSystem( string & sysname, string & system)
 {
 	Systems.insert( sysname, system);
 }
-	
+
+string			ZoneMgr::getSystem( string & name)
+{
+	return Systems.get( name);
+}
 
 StarSystem *	ZoneMgr::addZone( string starsys)
 {
 	COUT<<">>> ADDING A NEW ZONE = "<<starsys<<" - # OF ZONES = "<<_Universe->star_system.size()<<endl;
     ClientWeakList* lst = new ClientWeakList;
-	list<Unit *> ulst;
+	//list<Unit *> ulst;
 	StarSystem * sts=NULL;
 	// Generate the StarSystem
 	string starsysfile = starsys+".system";
-	//UniverseUtil::ComputeSystemSerials( starsys);
+	UniverseUtil::ComputeSystemSerials( starsysfile);
 	//sts = new StarSystem( starsysfile.c_str(), Vector(0,0,0));
 	//_Universe->Generate2( sts);
 	sts = _Universe->GenerateStarSystem (starsysfile.c_str(),"",Vector(0,0,0));
 	zone_list.push_back( lst);
-	zone_unitlist.push_back( ulst);
+	//zone_unitlist.push_back( ulst);
 	// Add zero as number of clients in zone since we increment in ZoneMgr::addClient()
 	zone_clients.push_back( 0);
-	zone_units.push_back( 0);
+	//zone_units.push_back( 0);
 	COUT<<"<<< NEW ZONE ADDED - # OF ZONES = "<<_Universe->star_system.size()<<endl;
 	return sts;
 }
@@ -67,7 +74,6 @@ void	ZoneMgr::addClient( ClientWeakPtr clt, int zone)
 
 	lst->push_back( clt );
 	zone_clients[zone]++;
-	clt->zone = zone;
 	// Now we add the unit in that starsystem
 	sts->AddUnit( clt->game_unit);
 }
@@ -77,16 +83,19 @@ void	ZoneMgr::addClient( ClientWeakPtr clt, int zone)
 /**** addUnit                                                                               *****/
 /************************************************************************************************/
 
+/*
 void	ZoneMgr::addUnit( Unit * un, int zone)
 {
 	zone_unitlist[zone].push_back( un);
 	zone_units[zone]++;
 }
+*/
 
 /************************************************************************************************/
 /**** removeUnit                                                                            *****/
 /************************************************************************************************/
 
+/*
 void	ZoneMgr::removeUnit( Unit * un, int zone)
 {
 	if( zone_unitlist[zone].empty())
@@ -97,17 +106,28 @@ void	ZoneMgr::removeUnit( Unit * un, int zone)
 	zone_unitlist[zone].remove( un);
 	zone_units[zone]--;
 }
+*/
 
 // Returns NULL if no corresponding Unit was found
 Unit *	ZoneMgr::getUnit( ObjSerial unserial, unsigned short zone)
 {
-	LUI i;
 	Unit * un = NULL;
+	UnitCollection::UnitIterator iter = (_Universe->star_system[zone]->getUnitList()).createIterator();
+
+	// Clients not ingame are removed from the drawList so it is ok not to test that
+	while( (un=iter.current()) != NULL)
+	{
+		if( un->GetSerial() == unserial)
+			break;
+		iter.advance();
+	}
+	/*
 	for( i=zone_unitlist[zone].begin(); i!=zone_unitlist[zone].end(); i++)
 	{
 		if( (*i)->GetSerial()==unserial)
 			un = (*i);
 	}
+	*/
 
 	return un;
 }
@@ -125,8 +145,8 @@ StarSystem *	ZoneMgr::addClient( ClientWeakPtr cltw, string starsys, unsigned sh
 	StarSystem * sts=NULL;
 	StarSystem * ret=NULL;
 
-	if( !(ret = sts = _Universe->getStarSystem( starsys)))
-	//if( !(ret = sts = GetLoadedStarSystem( starsys)))
+	string sysfile = starsys+".system";
+	if( !(ret = sts = GetLoadedStarSystem( sysfile.c_str())))
 	{
 		// Add a network zone (StarSystem equivalent) and create the new StarSystem
 		// StarSystem is not loaded so we generate it
@@ -135,6 +155,7 @@ StarSystem *	ZoneMgr::addClient( ClientWeakPtr cltw, string starsys, unsigned sh
 		// It also mean that there is nobody in that system so no need to send update
 		// Return false since the starsystem didn't contain any client
 		num_zone = _Universe->star_system.size()-1;
+		sts->SetZone( _Universe->StarSystemIndex( sts));
 	}
 	else
 	{
@@ -151,11 +172,11 @@ StarSystem *	ZoneMgr::addClient( ClientWeakPtr cltw, string starsys, unsigned sh
         zone_list[num_zone] = lst = new ClientWeakList;
 	lst->push_back( cltw );
     ClientPtr clt(cltw);
-	clt->zone = num_zone;
+	// Not needed : zone is an attribute of StarSystem
+	//clt->game_unit.GetUnit()->SetZone( num_zone);
 	zone_clients[num_zone]++;
-	cerr<<zone_clients[clt->zone]<<" clients now in zone "<<clt->zone<<endl;
+	cerr<<zone_clients[num_zone]<<" clients now in zone "<<num_zone<<endl;
 
-	// Compute a safe entrance point -> DONE WHEN LOGIN ACCEPTED
 	//QVector safevec;
 	sts->AddUnit( clt->game_unit.GetUnit());
 	return ret;
@@ -170,18 +191,26 @@ void	ZoneMgr::removeClient( ClientPtr clt )
 {
 	StarSystem * sts;
 	Unit * un = clt->game_unit.GetUnit();
-    ClientWeakList* lst = zone_list[clt->zone];
+	unsigned short zonenum = un->activeStarSystem->GetZone();
+    ClientWeakList* lst = zone_list[zonenum];
 
 	if( lst == NULL || lst->empty() )
 	{
         cerr<<"Trying to remove client on an empty list !!"<<endl;
         exit( 1);
     }
-
-	lst->remove( clt );
-	zone_clients[clt->zone]--;
-	cerr<<zone_clients[clt->zone]<<" clients left in zone "<<clt->zone<<endl;
-	sts = _Universe->star_system[clt->zone];
+	for (ClientWeakList::iterator q = lst->begin();
+		q!=lst->end();
+		++q) {
+			ClientWeakPtr cwp = *q;
+			ClientWeakPtr ocwp (clt);
+			if ((!(cwp<ocwp))&&!(ocwp<cwp)) {
+				lst->erase(q);
+			}
+		}
+	zone_clients[zonenum]--;
+	cerr<<zone_clients[zonenum]<<" clients left in zone "<<zonenum<<endl;
+	sts = _Universe->star_system[zonenum];
 	sts->RemoveUnit( un);
 	// SHIP MAY NOT HAVE BEEN KILLED BUT JUST CHANGED TO ANOTHER STAR SYSTEM -> NO KILL
 	//un->Kill();
@@ -202,17 +231,18 @@ void ZoneMgr::broadcast( ClientWeakPtr cltw, Packet * pckt )
     ClientPtr clt( cltw );
 	Unit * un = clt->game_unit.GetUnit();
 	Unit * un2 = NULL;
-    if( clt->zone > zone_list.size() )
+	unsigned short zonenum = un->activeStarSystem->GetZone();
+    if( zonenum > zone_list.size() )
     {
-        cerr<<"Trying to send update to nonexistant zone " << clt->zone << endl;
+        cerr<<"Trying to send update to nonexistant zone " << zonenum << endl;
         return;
     }
 
-    // cout<<"Sending update to "<<(zone_list[clt->zone].size()-1)<<" clients"<<endl;
-    ClientWeakList* lst = zone_list[clt->zone];
+    // cout<<"Sending update to "<<(zone_list[zonenum].size()-1)<<" clients"<<endl;
+    ClientWeakList* lst = zone_list[zonenum];
     if( lst == NULL )
     {
-        cerr<<"Trying to send update to nonexistant zone " << clt->zone << endl;
+        cerr<<"Trying to send update to nonexistant zone " << zonenum << endl;
         return;
     }
 
@@ -227,8 +257,7 @@ void ZoneMgr::broadcast( ClientWeakPtr cltw, Packet * pckt )
         {
             COUT << "BROADCASTING " << pckt->getCommand()
                  << " to client n "<<un2->GetSerial() << endl;
-            pckt->setNetwork( &cpi->cltadr, cpi->sock);
-            pckt->bc_send( );
+            pckt->bc_send( cpi->cltadr, cpi->sock);
         }
     }
 }
@@ -240,7 +269,6 @@ void ZoneMgr::broadcast( ClientWeakPtr cltw, Packet * pckt )
 // Broadcast a packet to a zone clients with its serial as argument
 void	ZoneMgr::broadcast( int zone, ObjSerial serial, Packet * pckt )
 {
-    // COUT<<"Sending update to "<<(zone_list[clt->zone]->size()-1)<<" clients"<<endl;
     ClientWeakList* lst = zone_list[zone];
     if( lst == NULL ) return;
 
@@ -254,8 +282,7 @@ void	ZoneMgr::broadcast( int zone, ObjSerial serial, Packet * pckt )
 		{
 			COUT<<"Sending update to client n "<< clt->game_unit.GetUnit()->GetSerial();
 			COUT<<endl;
-			pckt->setNetwork( &clt->cltadr, clt->sock);
-			pckt->bc_send( );
+			pckt->bc_send( clt->cltadr, clt->sock);
 		}
 	}
 }
@@ -263,7 +290,6 @@ void	ZoneMgr::broadcast( int zone, ObjSerial serial, Packet * pckt )
 // Broadcast a packet to a zone clients with its serial as argument but not to the originating client
 void	ZoneMgr::broadcastNoSelf( int zone, ObjSerial serial, Packet * pckt )
 {
-    // COUT<<"Sending update to "<<(zone_list[clt->zone]->size()-1)<<" clients"<<endl;
     ClientWeakList* lst = zone_list[zone];
     if( lst == NULL ) return;
 
@@ -277,8 +303,7 @@ void	ZoneMgr::broadcastNoSelf( int zone, ObjSerial serial, Packet * pckt )
 		{
 			COUT<<"Sending update to client n "<< clt->game_unit.GetUnit()->GetSerial();
 			COUT<<endl;
-			pckt->setNetwork( &clt->cltadr, clt->sock);
-			pckt->bc_send( );
+			pckt->bc_send( clt->cltadr, clt->sock);
 		}
 	}
 }
@@ -290,7 +315,6 @@ void	ZoneMgr::broadcastNoSelf( int zone, ObjSerial serial, Packet * pckt )
 // Broadcast a packet to a zone clients with its serial as argument
 void	ZoneMgr::broadcastSample( int zone, ObjSerial serial, Packet * pckt, float frequency )
 {
-    // COUT<<"Sending update to "<<(zone_list[clt->zone]->size()-1)<<" clients"<<endl;
     ClientWeakList* lst = zone_list[zone];
 	Unit * un;
     if( lst == NULL ) return;
@@ -308,8 +332,7 @@ void	ZoneMgr::broadcastSample( int zone, ObjSerial serial, Packet * pckt, float
 		{
 			COUT<<"Sending sound sample to client n "<<clt->game_unit.GetUnit()->GetSerial();
 			COUT<<endl;
-			pckt->setNetwork( &clt->cltadr, clt->sock);
-			pckt->bc_send( );
+			pckt->bc_send( clt->cltadr, clt->sock);
 		}
 	}
 }
@@ -321,7 +344,6 @@ void	ZoneMgr::broadcastSample( int zone, ObjSerial serial, Packet * pckt, float
 // Broadcast a packet to a zone clients with its serial as argument
 void	ZoneMgr::broadcastText( int zone, ObjSerial serial, Packet * pckt, float frequency )
 {
-    // COUT<<"Sending update to "<<(zone_list[clt->zone]->size()-1)<<" clients"<<endl;
     ClientWeakList* lst = zone_list[zone];
 	Unit * un;
     if( lst == NULL ) return;
@@ -339,8 +361,7 @@ void	ZoneMgr::broadcastText( int zone, ObjSerial serial, Packet * pckt, float fr
 		{
 			COUT<<"Sending sound sample to client n "<<clt->game_unit.GetUnit()->GetSerial();
 			COUT<<endl;
-			pckt->setNetwork( &clt->cltadr, clt->sock);
-			pckt->bc_send( );
+			pckt->bc_send( clt->cltadr, clt->sock);
 		}
 	}
 }
@@ -350,13 +371,17 @@ void	ZoneMgr::broadcastText( int zone, ObjSerial serial, Packet * pckt, float fr
 /************************************************************************************************/
 
 // Broadcast all positions
+// This function sends interpolated and predicted positions based on the "semi-ping" between the sender clients and the server
+// the receiver client will only have to interpolate and predict on his own "semi-ping" value
 void	ZoneMgr::broadcastSnapshots( bool update_planets)
 {
 	unsigned int i=0;
-	CWLI k, l;
-	LUI m;
-	NetBuffer netbuf;
+	CWLI k;
 
+	for( i=0; i<_Universe->star_system.size(); i++)
+	{
+
+	}
 	//COUT<<"Sending snapshot for ";
 	//int h_length = Packet::getHeaderLength();
 	// Loop for all systems/zones
@@ -365,19 +390,53 @@ void	ZoneMgr::broadcastSnapshots( bool update_planets)
 		// Check if system contains player(s)
 		if( zone_clients[i]>0)
 		{
-			int	nbclients = 0, nbunits=0;
-			Packet pckt;
-
-			cerr<<"BROADCAST SNAPSHOTS = "<<zone_clients[i]<<" clients in zone "<<i<<endl;
+			COUT << "BROADCAST SNAPSHOTS = "<<zone_clients[i]<<" clients in zone "<<i<<endl;
 			// Loop for all the zone's clients
 			for( k=zone_list[i]->begin(); k!=zone_list[i]->end(); k++)
 			{
-			/************************* START CLIENTS BROADCAST ***************************/
-				// If we don't want to send a client its own info set nbclients to zone_clients-1 for memory saving (ok little)
-                if( (*k).expired() ) continue;
                 ClientPtr cltk( *k );
-				if( cltk->ingame )
+				// If that client is ingame we send to it position info
+				if( cltk->ingame==true)
 				{
+				    int       nbunits=0;
+			        Packet    pckt;
+                    NetBuffer netbuf;
+
+                    COUT << "CLEAN NETBUF" << endl;
+
+					// This also means clients will receive info about themselves
+                    // which they should ignore or take into account sometimes
+                    // (according to their ping value)
+					UnitCollection::UnitIterator iter = (_Universe->star_system[i]->getUnitList()).createIterator();
+					Unit* unit;
+
+					// Add the client we send snapshot to its own deltatime (semi-ping)
+					netbuf.addFloat( cltk->getDeltatime() );
+                    COUT << "   *** deltatime " << cltk->getDeltatime() << endl;
+					// Clients not ingame are removed from the drawList so it is ok not to test that
+					while( (unit=iter.current()) != NULL)
+					{
+						// Only send unit that ate UNITPTR and PLANETPTR+NEBULAPTR if update_planets
+						if( (unit->isUnit()==UNITPTR || unit->isUnit()==ASTEROIDPTR || unit->isUnit()==MISSILEPTR) || ((unit->isUnit()==PLANETPTR || unit->isUnit()==NEBULAPTR) && update_planets) )
+						{
+							ClientState cs( unit);
+
+							/* TEST if it is a client and then set deltatime */
+							//ClientPtr cltl;
+							//if( (cltl = Server->getClientFromSerial( unit->GetSerial())))
+							//{
+								//cs.setDelay( cltl->getDeltatime() );
+								// This should be moved out of the 'if' when download manager is working
+							//}
+							bool added = addPosition( netbuf, unit, cs);
+                            if( added )
+							    nbunits++;
+						}
+						iter.advance();
+					}
+			/************************* START CLIENTS BROADCAST ***************************/
+				/*
+				// If we don't want to send a client its own info set nbclients to zone_clients-1 for memory saving (ok little)
 					nbclients = zone_clients[i]-1;
 					netbuf.Reset();
 					for( l=zone_list[i]->begin(); l!=zone_list[i]->end(); l++)
@@ -390,14 +449,18 @@ void	ZoneMgr::broadcastSnapshots( bool update_planets)
 							// Create a client state with a delta time
 							ClientState cs( un);
 							// HAVE TO VERIFY WHICH DELTATIME IS TO BE SENT
-							cs.setDelay( cltl->deltatime);
-							this->addPosition( netbuf, un, cltk->game_unit.GetUnit(), cs);
+							cs.setDelay( cltl->getDeltatime());
+							bool added = addPosition( netbuf, un, cs);
+                            if( added )
+							    nbunits++;
 						}
 						// Else : always send back to clients their own info or just ignore ?
 						// Ignore for now
 					}
+					*/
 			/************************* END CLIENTS BROADCAST ***************************/
 			/************************* START UNITS BROADCAST ***************************/
+				/*
 					nbunits = zone_units[i];
 					cerr<<"BROADCAST SNAPSHOTS = "<<zone_units[i]<<" units in zone "<<i<<endl;
 					//netbuf.Reset();
@@ -408,17 +471,20 @@ void	ZoneMgr::broadcastSnapshots( bool update_planets)
 						{
 							// Create a client state with a delta time too ?? WHICH ONE ???
 							ClientState cs( (*m));
-							this->addPosition( netbuf, (*m), cltk->game_unit.GetUnit(), cs);
+							bool added = addPosition( netbuf, (*m), cs);
+                            if( added )
+							    nbunits++;
 							// Else : always send back to clients their own info or just ignore ?
 							// Ignore for now
 						}
 					}
+				*/
 			/************************* END UNITS BROADCAST ***************************/
 					// Send snapshot to client k
-					if(netbuf.getDataLength()>0)
+					if(nbunits>0)
 					{
 						//COUT<<"\tsend update for "<<(p+j)<<" clients"<<endl;
-						pckt.send( CMD_SNAPSHOT, nbclients+nbunits,
+						pckt.send( CMD_SNAPSHOT, /*nbclients+*/nbunits,
                                    netbuf.getData(), netbuf.getDataLength(),
                                    SENDANDFORGET, &(cltk->cltadr), cltk->sock,
                                    __FILE__, PSEUDO__LINE__(337) );
@@ -429,10 +495,11 @@ void	ZoneMgr::broadcastSnapshots( bool update_planets)
 	}
 }
 
-void	ZoneMgr::addPosition( NetBuffer & netbuf, Unit * un, Unit * clt_unit, ClientState & un_cs)
+bool ZoneMgr::addPosition( NetBuffer & netbuf, Unit * un, ClientState & un_cs)
 {
 	// This test may be wrong for server side units -> may cause more traffic than needed
-	if( !(un->old_state.getPosition()==un->curr_physical_state.position) || !(un->old_state.getOrientation()==un->curr_physical_state.orientation))
+	if( 1 /* !(un->old_state.getPosition()==un->curr_physical_state.position) ||
+             !(un->old_state.getOrientation()==un->curr_physical_state.orientation) */ )
 	{
 		// Unit 'un' can see Unit 'iter'
 		// For now only check if the 'iter' client is in front of Unit 'un')
@@ -444,26 +511,46 @@ void	ZoneMgr::addPosition( NetBuffer & netbuf, Unit * un, Unit * clt_unit, Clien
 			//ratio = radius/distance;
 			if( 1 /* ratio > XX client not too far */)
 			{
+                COUT << "   *** FullUpdate ser=" << un->GetSerial() << " cs=" << un_cs << endl;
 				// Mark as position+orientation+velocity update
-				netbuf.addChar( CMD_FULLUPDATE);
+				netbuf.addChar( ZoneMgr::FullUpdate );
 				netbuf.addShort( un->GetSerial());
 				// Put the current client state in
 				netbuf.addClientState( un_cs);
-				un_cs.display();
 				// Increment the number of clients we send full info about
 			}
 			// Here find a condition for which sending only position would be enough
 			else if( 0 /* ratio>=1 far but still visible */)
 			{
+                COUT << "   *** PosUpdate ser=" << un->GetSerial()
+                     << " pos=" << un_cs.getPosition().i
+                     << "," << un_cs.getPosition().j
+                     << "," << un_cs.getPosition().k << endl;
 				// Mark as position update only
-				netbuf.addChar( CMD_POSUPDATE);
+				netbuf.addChar( ZoneMgr::PosUpdate );
 				// Add the client serial
 				netbuf.addShort( un->GetSerial());
 				netbuf.addQVector( un_cs.getPosition());
 				// Increment the number of clients we send limited info about
 			}
+            else
+            {
+                COUT << "Client counted but not sent because of ratio!" << endl;
+                return false;
+            }
 		}
+        else
+        {
+            COUT << "Client counted but not sent because of distance!" << endl;
+            return false;
+        }
 	}
+    else
+    {
+        COUT << "Client counted but not sent because of position/orientation test!" << endl;
+        return false;
+    }
+    return true;
 }
 
 /************************************************************************************************/
@@ -474,8 +561,7 @@ void	ZoneMgr::addPosition( NetBuffer & netbuf, Unit * un, Unit * clt_unit, Clien
 void	ZoneMgr::broadcastDamage( )
 {
 	unsigned int i=0;
-	CWLI k, l;
-	LUI m;
+	CWLI k;
 	NetBuffer netbuf;
 
 	//COUT<<"Sending snapshot for ";
@@ -492,7 +578,6 @@ void	ZoneMgr::broadcastDamage( )
 			// client.
 			int	nbclients = 0, nbunits=0;
 			Packet pckt;
-			Unit * un;
 
 			//cerr<<"BROADCAST DAMAGE = "<<zone_clients[i]<<" clients in zone "<<i<<endl;
 			// Loop for all the zone's clients
@@ -502,7 +587,22 @@ void	ZoneMgr::broadcastDamage( )
                 ClientPtr cp( *k );
 				if( cp->ingame )
 				{
+					UnitCollection::UnitIterator iter = (_Universe->star_system[i]->getUnitList()).createIterator();
+					Unit * unit;
+
+					// Clients not ingame are removed from the drawList so it is ok not to test that
+					while( (unit=iter.current()) != NULL)
+					{
+						if( unit->damages)
+						{
+							this->addDamage( netbuf, unit);
+							nbunits++;
+						}
+
+						iter.advance();
+					}
 			/************************* START CLIENTS BROADCAST ***************************/
+					/*
 					nbclients = zone_clients[i];
 					netbuf.Reset();
 					for( l=zone_list[i]->begin(); l!=zone_list[i]->end(); l++)
@@ -515,8 +615,10 @@ void	ZoneMgr::broadcastDamage( )
 						if( cltl->ingame && un && un->damages)
 							this->addDamage( netbuf, un);
 					}
+					*/
 			/************************* END CLIENTS BROADCAST ***************************/
 			/************************* START UNITS BROADCAST ***************************/
+					/*
 					//cerr<<"BROADCAST DAMAGE = "<<zone_units[i]<<" units in zone "<<i<<endl;
 					nbunits = zone_units[i];
 					//netbuf.Reset();
@@ -526,10 +628,12 @@ void	ZoneMgr::broadcastDamage( )
 						if( (*m)->damages)
 							this->addDamage( netbuf, (*m));
 					}
+					*/
+
 					// Send snapshot to client k
 					if( netbuf.getDataLength() > 0)
 					{
-						pckt.send( CMD_SNAPDAMAGE, nbclients+nbunits,
+						pckt.send( CMD_SNAPDAMAGE, /*nbclients+*/nbunits,
                                    netbuf.getData(), netbuf.getDataLength(),
                                    SENDANDFORGET, &(cp->cltadr), cp->sock,
                                    __FILE__, PSEUDO__LINE__(442) );
@@ -642,7 +746,8 @@ void ZoneMgr::sendZoneClients( ClientWeakPtr clt )
     if( clt.expired() ) return;
     ClientPtr cp( clt );
 
-    ClientWeakList* lst = zone_list[cp->zone];
+    unsigned short cltzone = cp->game_unit.GetUnit()->activeStarSystem->GetZone();
+    ClientWeakList* lst = zone_list[cltzone];
     if( lst == NULL )
     {
 	    COUT << "\t>>> WARNING: Did not send info about " << nbclients << " other ships to client serial " << cp->game_unit.GetUnit()->GetSerial() << " because of empty (inconsistent?) zone" << endl;
@@ -691,13 +796,14 @@ int		ZoneMgr::getZoneClients( ClientWeakPtr clt, char * bufzone)
     if( clt.expired() ) return 0;
     ClientPtr cp( clt );
 
-	nbt = zone_clients[cp->zone];
+	unsigned short cltzone = cp->game_unit.GetUnit()->activeStarSystem->GetZone();
+	nbt = zone_clients[cltzone];
 	NetBuffer netbuf;
 
-	COUT<<"ZONE "<<cp->zone<<" - "<<nbt<<" clients"<<endl;
+	COUT<<"ZONE "<<cltzone<<" - "<<nbt<<" clients"<<endl;
 	netbuf.addShort( nbt);
-    assert( zone_list[cp->zone] != NULL );
-	for( k=zone_list[cp->zone]->begin(); k!=zone_list[cp->zone]->end(); k++)
+    assert( zone_list[cltzone] != NULL );
+	for( k=zone_list[cltzone]->begin(); k!=zone_list[cltzone]->end(); k++)
 	{
 		COUT<<"SENDING : ";
         if( (*k).expired() ) continue;
@@ -740,7 +846,7 @@ void	ZoneMgr::displayStats()
 	{
 		cout<<"\t\tStar system "<<i<<" = \"<<_Universe->star_system[i]->getName()"<<"\""<<endl;
 		cout<<"\t\t\tNumber of clients :\t"<<zone_clients[i]<<endl;
-		cout<<"\t\t\tNumber of units :\t"<<zone_units[i]<<endl;
+		//cout<<"\t\t\tNumber of units :\t"<<zone_units[i]<<endl;
 	}
 }
 
@@ -760,7 +866,7 @@ int		ZoneMgr::displayMemory()
 		
 		cout<<"\t\tStar system "<<i<<" = \"<<_Universe->star_system[i]->getName()"<<"\""<<endl;
 		memclient = zone_clients[i]*sizeof( class Client);
-		memunit = zone_units[i]*sizeof( class Unit);
+		//memunit = zone_units[i]*sizeof( class Unit);
 		memvars = zone_clients[i]*sizeof( int)*2;
 		cout<<"\t\t\tMemory for clients :\t"<<(memclient/1024)<<" KB ("<<memclient<<" bytes)"<<endl;
 		cout<<"\t\t\tMemory for units :\t"<<(memunit/1024)<<" KB ("<<memunit<<" bytes)"<<endl;
