@@ -5,7 +5,7 @@
 #include "gfx/vec.h"
 #include "cmd/unit_generic.h"
 #include "python_class.h"
-#ifdef USE_BOOST_129
+#ifndef USE_BOOST_128
 #include <boost/python.hpp>
 #else
 #include <boost/python/objects.hpp>
@@ -28,4 +28,4 @@ void ExportUnit (boost::python::class_builder <UnitWrapper> &Class) {
 	ExportUnitSECOND (Class);
 	ExportUnitTHIRD (Class);
 
-}
\ No newline at end of file
+}
