@@ -1,7 +1,7 @@
 #include "cmd/unit_generic.h"
 #include "hashtable.h"
 #include <float.h>
-#include "vs_path.h"
+#include "vsfilesystem.h"
 #include <vector>
 #include <string>
 #include "configxml.h"
@@ -12,40 +12,59 @@
 #include "cmd/script/mission.h"
 #include "gfx/cockpit_generic.h"
 #include "networking/const.h"
-#include <sys/types.h>
-#include <sys/stat.h>
+#include "vsfilesystem.h"
 using namespace std;
-
+using namespace VSFileSystem;
+std::string CurrentSaveGameName="";
  std::string GetHelperPlayerSaveGame (int num) {
 
 if( Network==NULL)
 {
+	if (CurrentSaveGameName.length()>0){
+		VSFile f;
+		VSError err = f.OpenCreateWrite( "save.4.x.txt", UnknownFile);
+		if (err<=Ok) {
+			f.Write(CurrentSaveGameName);
+			f.Close();
+		}		
+		if (num!=0) {
+			return CurrentSaveGameName+XMLSupport::tostring(num);
+		}
+		return CurrentSaveGameName;
+	}
     cout << "Hi helper play " << num << endl;
   static string *res=NULL;
   if (res==NULL) {
     res = new std::string;
-    changehome(false);
     //char c[2]={'\0','\0'};
-    FILE * fp = fopen (("save.4.x.txt"),"r");
-    if (!fp) {
-      fp = fopen ("save.4.x.txt","w");
-      if (fp) {
-	fwrite ("defaultsavegame.4x\n",19,1,fp);
-	fclose (fp);
+	VSFile f;
+	// TRY TO OPEN THE save.4.x.txt FILE WHICH SHOULD CONTAIN THE NAME OF THE SAVE TO USE
+	VSError err = f.OpenReadOnly( "save.4.x.txt", UnknownFile);
+    if (err>Ok) {
+	  // IF save.4.x.txt DOES NOT EXIST WE CREATE ONE WITH "default" AS SAVENAME
+	  err = f.OpenCreateWrite( "save.4.x.txt", UnknownFile);
+      if (err<=Ok) {
+		f.Write("default\n",8);
+		f.Close();
       }
-      fp = fopen (("save.4.x.txt"),"r");
-    }
-    if (fp) {
-	  long length=0;
+	  else
 	  {
-		  struct stat st;
-		  if (fstat(fileno(fp),&st)==0)
-			length=st.st_size;
+	  	fprintf( stderr, "!!! ERROR : Creating default save.4.x.txt file : %s\n", f.GetFullPath().c_str());
+		exit(1);
 	  }
+	  err = f.OpenReadOnly( "save.4.x.txt", UnknownFile);
+	  if( err>Ok)
+	  {
+	  	fprintf( stderr, "!!! ERROR : Opening the default save we just created\n");
+		exit(1);
+	  }
+    }
+    if (err<=Ok) {
+	  long length=f.Size();
       if (length>0) {
       char * temp = (char *)malloc (length+1);
       temp[length]='\0';
-      fread (temp,sizeof(char),length,fp);
+      f.Read (temp,length);
       bool end=true;
       for (int i=length-1;i>=0;i--) {
         if (temp[i]=='\r'||temp[i]=='\n') {
@@ -59,18 +78,18 @@ if( Network==NULL)
       *res = (temp);
       free (temp);
       }
-      fclose (fp);
+      f.Close();
 
     }
 #if 0
-    if (fp) {
-    while (!feof (fp)) {
-      c[0]=fgetc (fp);
-      if (!feof(fp)) {
+    if (err<=Ok) {
+    while (!f.Eof()) {
+      f.Read( &c[0], sizeof( char));
+      if (!f.Eof()) {
         if (c[0]!='\r'&&c[0]!='\n'&&c[0]!='\0') {
           if (c[0]==' ') {
             c[0]='_';
-            if (feof(fp)) {
+            if (f.Eof()) {
               continue;
             }
           }
@@ -80,30 +99,36 @@ if( Network==NULL)
         break;
       }
     }
-    fclose (fp);
+    f.Close();
 #endif
-    if (!res->empty()) {
-      if (*res->begin()=='~') {
-	fp = fopen (("save.4.x.txt"),"w");
-	if (fp) {
-	  for (unsigned int i=1;i<res->length();i++) {
-	    fputc (*(res->begin()+i),fp);
+    if (!res->empty())
+	{
+	  // Set filetype to Unknown so that it is searched in homedir/
+      if (*res->begin()=='~')
+	  {
+		  err = f.OpenCreateWrite( "save.4.x.txt", VSFileSystem::UnknownFile);
+		if (err<=Ok) {
+	  	for (unsigned int i=1;i<res->length();i++)
+		{
+			char cc = *(res->begin()+i);
+	    	f.Write ( &cc,sizeof(char));
+	  	}
+	 	char cc=0;
+	  	f.Write (&cc,sizeof(char));
+	  	f.Close();
 	  }
-	  fputc ('\0',fp);
-	  fclose (fp);
-	}
-      }
     }
+  } 
 
     
 #if 0
-    fp = fopen (("save.4.x.txt"),"w");
-    if (fp) {
-      fputc('\0',fp);
-      fclose (fp);
+	err = f.OpenReadOnly( "save.4.x.txt", SaveFile);
+    if (err<=Ok) {
+	  char cc=0;
+      f.Write(&cc,sizeof( char));
+      f.Close();
     }
 #endif
-    returnfromhome();
   }
   if (num==0||res->empty()) {
     cout << "Here";
@@ -139,27 +164,25 @@ std::string GetReadPlayerSaveGame(int num) {
   return ret;
 }
 
-void FileCopy (const char * src, const char * dst) {
+// Used only to copy a savegame to a different named one
+void SaveFileCopy (const char * src, const char * dst) {
   if (dst[0]!='\0'&&src[0]!='\0') {
 
-  FILE * fp = fopen (src,"r");
-  if (fp) {
-	  long length=0;
-	  {
-	      struct stat st;
-	      if (fstat(fileno(fp), &st)==0)
-	        length=st.st_size;
-	  }
-      char * info = new char [length];
-      fread(info,length,sizeof(char),fp);
-      fclose (fp);
-      fp = fopen (dst,"w");
-      if (fp) {
-        fwrite (info,length,sizeof(char),fp);
-        fclose(fp);
+  VSFile f;
+  VSError err = f.OpenReadOnly( src, SaveFile);
+  if (err<=Ok) {
+      string savecontent = f.ReadFull();
+	  f.Close();
+      err = f.OpenCreateWrite( dst, SaveFile);
+      if (err<=Ok) {
+		f.Write( savecontent);
+	    f.Close();
       }
-      delete [] info;
+	  else
+	  	fprintf( stderr, "WARNING : couldn't open savegame to copy to : %s as SaveFile", dst);
   }
+  else
+  	fprintf(stderr, "WARNING : couldn't find the savegame to copy : %s as SaveFile", src);
   }
 }
 class MissionStringDat {
@@ -195,11 +218,11 @@ string SaveGame::GetOldStarSystem () {
 }
 
 void SaveGame::SetPlayerLocation (const QVector &v) {
-  //fprintf (stderr,"Set Location %f %f %f",v.i,v.j,v.k);
+  //VSFileSystem::vs_fprintf (stderr,"Set Location %f %f %f",v.i,v.j,v.k);
   if ((FINITE (v.i)&&FINITE(v.j)&&FINITE(v.k))) {
     PlayerLocation =v;
   }else {
-    fprintf (stderr,"ERROR saving unit");
+    VSFileSystem::vs_fprintf (stderr,"ERROR saving unit");
     PlayerLocation.Set(1,1,1);
   }
 }
@@ -243,7 +266,7 @@ string SaveGame::WriteNewsData () {
   return ret;
 }
 vector <string> parsePipedString(string s) {
-  unsigned int loc;
+  string::size_type loc;
   vector <string> ret;
   while ((loc = s.find("|"))!=string::npos) {
     ret.push_back( s.substr (0,loc));
@@ -278,8 +301,35 @@ void WriteSaveGame (Cockpit * cp,bool auto_save) {
     if (GetWritePlayerSaveGame(player_num).length()&&!auto_save) {
       cp->savegame->SetSavedCredits (_Universe->AccessCockpit()->credits);
       cp->savegame->SetStarSystem(cp->activeStarSystem->getFileName());
-      un->WriteUnit(GetWritePlayerSaveGame(player_num).c_str());
+      //      un->WriteUnit(GetWritePlayerSaveGame(player_num).c_str());
       cp->savegame->SetPlayerLocation(un->LocalPosition());    
+      for (int i=0;i<cp->unitfilename.size();i+=2) {
+        if (i==2) i=1;
+        VSFile src,dst;
+        string srcnam=cp->GetUnitModifications();
+
+        VSError e = src.OpenReadOnly(srcnam+"/"+cp->unitfilename[i]+".csv",UnitSaveFile);
+        if (e<=Ok) {
+          string dstnam= GetWritePlayerSaveGame(player_num);
+          VSFileSystem::CreateDirectoryHome (VSFileSystem::savedunitpath+"/"+dstnam);
+          VSError f = dst.OpenCreateWrite(dstnam+"/"+cp->unitfilename[i]+".csv",UnitFile);
+          if (f<=Ok) {
+            string srcdata=src.ReadFull();
+            dst.Write(srcdata);
+          }else {
+            printf ("Error: Cannot Copy Unit %s from save file %s to %s\n",
+                    cp->unitfilename[i].c_str(),
+                    srcnam.c_str(),
+                    dstnam.c_str());
+                    
+          }
+        }else {
+            printf ("Error: Cannot Open Unit %s from save file %s.\n",
+                    cp->unitfilename[i].c_str(),
+                    srcnam.c_str());
+                    
+        }        
+      }
     }
   }
 
@@ -516,7 +566,7 @@ void SaveGame::ReadStardate( char * &buf)
 {
 	string stardate( AnyStringScanInString( buf));
 	cout<<"Read stardate : "<<stardate<<endl;
-	_Universe->current_stardate.Init( stardate);
+	_Universe->current_stardate.InitTrek( stardate);
 }
 
 void SaveGame::ReadSavedPackets (char * &buf) {
@@ -601,7 +651,7 @@ string SaveGame::WriteDynamicUniverse()
 	if( SERVER)
 	{
 		cerr<<"SAVING STARDATE - SERVER="<<SERVER<<endl;
-		dyn_univ += "\n0 stardate data"+AnyStringWriteString( _Universe->current_stardate.GetFullCurrentStarDate());
+		dyn_univ += "\n0 stardate data"+AnyStringWriteString( _Universe->current_stardate.GetFullTrekDate());
 	}
     memset( tmp, 0, MB);
     sprintf (tmp,"\n%d %s %s",0,"mission","data ");
@@ -636,27 +686,30 @@ string SaveGame::WriteDynamicUniverse()
 	return dyn_univ;
 }
 
+using namespace VSFileSystem;
+
 string SaveGame::WriteSaveGame (const char *systemname, const QVector &FP, float credits, std::vector<std::string> unitname, int player_num, std::string fact, bool write) {
   savestring = string("");
   if (outputsavegame.length()!=0) {
     printf ("Writing Save Game %s",outputsavegame.c_str());
-    changehome();
-    vschdir ("save");
 
 	savestring += WritePlayerData( FP, unitname, systemname, credits, fact);
 	savestring += WriteDynamicUniverse();
 
     if( write){
-	FILE * fp = fopen (outputsavegame.c_str(),"wb");
-		fwrite( savestring.c_str(), sizeof( char), savestring.length(), fp);
-		fclose (fp);
-		if (player_num!=-1) {
-		  last_pickled_data =last_written_pickled_data;
-		  FileCopy (outputsavegame.c_str(),GetWritePlayerSaveGame(player_num).c_str());
-		}
+	VSFile f;
+	VSError err = f.OpenCreateWrite( outputsavegame, SaveFile);
+	// WRITE THE SAVEGAME TO THE MISSION SAVENAME
+	f.Write( savestring.c_str(), savestring.length());
+	f.Close();
+	if (player_num!=-1) {
+			// AND THEN COPY IT TO THE SPECIFIED SAVENAME (from save.4.x.txt)
+          last_pickled_data =last_written_pickled_data;
+          string sg =GetWritePlayerSaveGame(player_num);
+          SaveFileCopy (outputsavegame.c_str(),sg.c_str());
+          
+	}
     }
-    vscdup();
-    returnfromhome();
 
   }
   return savestring;
@@ -664,57 +717,47 @@ string SaveGame::WriteSaveGame (const char *systemname, const QVector &FP, float
 
 static float savedcredits=0;
 float SaveGame::GetSavedCredits () {
-  return savedcredits;
+		return savedcredits;
 }
 void SaveGame::SetSavedCredits (float c) {
-  savedcredits = c;
+		savedcredits = c;
 }
 
 void SaveGame::ParseSaveGame (string filename, string &FSS, string originalstarsystem, QVector &PP, bool & shouldupdatepos,float &credits, vector <string> &savedstarship, int player_num, string str, bool read) {
-  char *tempfullbuf=0;
-  int tempfulllength=2048;
-  int readlen=0;
-  if (filename.length()>0)
-    filename=callsign+filename;
-  shouldupdatepos=!(PlayerLocation.i==FLT_MAX||PlayerLocation.j==FLT_MAX||PlayerLocation.k==FLT_MAX);
-  outputsavegame=filename;
-  changehome();
-  vschdir ("save");
-  FILE * fp = NULL;
-  if( read)
-  {
-	  if (filename.length()>0) {
-		if (GetReadPlayerSaveGame(player_num).length()) {
-			  fp = fopen (GetReadPlayerSaveGame(player_num).c_str(),"r");
-		}else {
-		  fp = fopen (filename.c_str(),"r");
+	char *tempfullbuf=0;
+	int tempfulllength=2048;
+	int readlen=0;
+	if (filename.length()>0)
+			filename=callsign+filename;
+	shouldupdatepos=!(PlayerLocation.i==FLT_MAX||PlayerLocation.j==FLT_MAX||PlayerLocation.k==FLT_MAX);
+	// WE WILL ALWAYS SAVE THE CURRENT SAVEGAME IN THE MISSION SAVENAME (IT WILL BE COPIED TO THE SPECIFIED SAVENAME)
+	outputsavegame=filename;
+	VSFile f;
+	VSError err=FileNotFound;
+	if( read)
+	{
+		if (filename.length()>0) {
+				// TRY TO GET THE SPECIFIED SAVENAME TO LOAD
+				string plsave = GetReadPlayerSaveGame(player_num);
+				if (plsave.length()) {
+					err = f.OpenReadOnly( plsave, SaveFile);
+				}else {
+					// IF NONE SIMPLY LOAD THE MISSION DEFAULT ONE
+					err = f.OpenReadOnly( filename, SaveFile);
+				}
 		}
-	  }
-	  if( fp)
-	  {
+		if( err<=Ok)
 		{
-		  struct stat st;
-		  if (fstat(fileno(fp),&st)==0) {
-		    tempfulllength=st.st_size;
-		  } else {
-		    tempfulllength=0;
-		  }
+				str = savestring = f.ReadFull();
 		}
-      
-	    tempfullbuf = (char *)malloc (tempfulllength+1);
-	    tempfullbuf[tempfulllength]=0;
-	    fread( tempfullbuf, sizeof( char), tempfulllength, fp);
-	    str = savestring = string( tempfullbuf);
-	  }
   }
-  vscdup();
-  returnfromhome();
-  if( fp || (!read && str!=""))
+  if( err<=Ok || (!read && str!=""))
   {
 	  savestring = str;
 	  if ( savestring.length()>0) {
 	    char * buf = new char[str.length()+1];
 		buf[str.length()]='\0';
+
 		memcpy( buf, str.c_str(), str.length());
 		char * deletebuf = buf;
 		char *tmp2= (char *)malloc(savestring.length()+2);
@@ -769,7 +812,7 @@ void SaveGame::ParseSaveGame (string filename, string &FSS, string originalstars
 		delete []deletebuf;
 	  }
 	  if( read)
-	  	fclose (fp);
+	  	f.Close();
   }
 
   if (PlayerLocation.i==FLT_MAX||PlayerLocation.j==FLT_MAX||PlayerLocation.k==FLT_MAX) {
@@ -787,8 +830,6 @@ void SaveGame::ParseSaveGame (string filename, string &FSS, string originalstars
 	FSS = ForceStarSystem;
   }
   SetSavedCredits(credits);
-  if (tempfullbuf)
-    free(tempfullbuf);
   //  return mysav;
 }
 
