@@ -4,7 +4,7 @@
 #include "cmd/planet_generic.h"
 #include "cmd/unit_factory.h"
 #include "vs_globals.h"
-#include "vs_path.h"
+#include "vsfilesystem.h"
 #include "configxml.h"
 #include "vegastrike.h"
 #include <assert.h>	/// needed for assert() calls.
@@ -34,7 +34,7 @@ extern void disableTerrainDraw( ContinuousTerrain *ct);
 AtmosphericFogMesh::AtmosphericFogMesh() {
 	er=eg=eb=ea=.5;
 	dr=dg=db=da=.5;	
-	meshname="sphereatm.xmesh";
+	meshname="sphereatm.bfxm";
 	scale=1.05;
 	focus=.5;
 	concavity=0;
@@ -289,18 +289,18 @@ void parse_dual_alpha (const char * alpha, BLENDFUNC & blendSrc, BLENDFUNC &blen
     }
 }
 
-int GetNumNearStarsScale (int currentnum) {
-  static float numstars=  XMLSupport::parse_float (vs_config->getVariable("graphics","num_near_stars_scale","2"));
-  return (int)(numstars*currentnum);
+int GetNumNearStarsScale () {
+  static float numstars=  XMLSupport::parse_float (vs_config->getVariable("graphics","num_near_stars","1000"));
+  return (int)(numstars);
 }
-int GetNumStarsScale (int currentnum) {
-  static float numstars=  XMLSupport::parse_float (vs_config->getVariable("graphics","num_far_stars_scale","2"));
-  return (int)(numstars*currentnum);
+int GetNumStarsScale () {
+  static float numstars=  XMLSupport::parse_float (vs_config->getVariable("graphics","num_far_stars","2000"));
+  return (int)(numstars);
 }
-float GetStarSpreadScale (float currentnum) {
-  static float numstars=  XMLSupport::parse_float (vs_config->getVariable("graphics","star_spread","200"));
+float GetStarSpreadScale () {
+  static float numstars=  XMLSupport::parse_float (vs_config->getVariable("graphics","star_spreading","30000"));
   static float gamespeed = XMLSupport::parse_float(vs_config->getVariable("physics","game_speed","1"));
-  return numstars*currentnum*gamespeed;
+  return numstars*gamespeed;
 }
 
 float ScaleOrbitDist (bool autogenerated) {
@@ -380,16 +380,9 @@ void StarSystem::beginElement(const string &name, const AttributeList &attribute
 	xml->backgroundname=(*iter).value;
 	break;
       case NEARSTARS:
-	xml->numnearstars = GetNumNearStarsScale(parse_int((*iter).value));
-	break;
       case STARS:
-	xml->numstars = GetNumStarsScale(parse_int ((*iter).value));
-	break;
       case STARSPREAD:
-	xml->starsp = GetStarSpreadScale(parse_float ((*iter).value));
-	break;
       case FADESTARS:
-	fprintf (stderr,"deprecated");
 	break;
       case NAME:
 	this->name = new char [strlen((*iter).value.c_str())+1];
@@ -407,7 +400,6 @@ void StarSystem::beginElement(const string &name, const AttributeList &attribute
       }
 
     }
-
     break;
   case RING:
     {
@@ -574,10 +566,10 @@ void StarSystem::beginElement(const string &name, const AttributeList &attribute
 			xml->fog.back().meshname=iter->value;
 			break;
 		case MINALPHA:
-			xml->fog.back().min_alpha=XMLSupport::parse_float(iter->value)*255;
+			xml->fog.back().min_alpha=(int)(XMLSupport::parse_float(iter->value)*255);
 			break;
 		case MAXALPHA:
-			xml->fog.back().max_alpha=XMLSupport::parse_float(iter->value)*255;
+			xml->fog.back().max_alpha=(int)(XMLSupport::parse_float(iter->value)*255);
 			break;
 		case CONCAVITY:
 			xml->fog.back().concavity=XMLSupport::parse_float(iter->value);
@@ -586,10 +578,10 @@ void StarSystem::beginElement(const string &name, const AttributeList &attribute
 			xml->fog.back().focus=XMLSupport::parse_float(iter->value);			
 			break;
 		case TAILMODESTART:
-			xml->fog.back().tail_mode_start=XMLSupport::parse_float(iter->value);
+			xml->fog.back().tail_mode_start=(int)(XMLSupport::parse_float(iter->value));
 			break;
 		case TAILMODEEND:
-			xml->fog.back().tail_mode_end=XMLSupport::parse_float(iter->value);
+			xml->fog.back().tail_mode_end=(int)(XMLSupport::parse_float(iter->value));
 			break;
 		case SCALEATMOS:
 			xml->fog.back().scale=XMLSupport::parse_float (iter->value);
@@ -701,7 +693,7 @@ void StarSystem::beginElement(const string &name, const AttributeList &attribute
 	if (p)
 	  p->setAtmosphere (a);
 	else
-	  fprintf (stderr,"atmosphere loose. no planet for it");
+	  VSFileSystem::vs_fprintf (stderr,"atmosphere loose. no planet for it");
       } 
       
     }
@@ -985,7 +977,6 @@ void StarSystem::beginElement(const string &name, const AttributeList &attribute
 	  radius = ScaleJumpRadius (radius);
     }
     radius*=xml->scale;
-	Unit * object;
     if (xml->unitlevel>2)
 	{
       assert(xml->moons.size()!=0);
@@ -1029,7 +1020,6 @@ void StarSystem::beginElement(const string &name, const AttributeList &attribute
 	break;
       case NAME:
 	fullname=(*iter).value;
-	cout << "\nFOUND unit name " << fullname << endl;
 	break;
       case XFILE:
 	delete []filename;
@@ -1244,38 +1234,41 @@ using namespace StarXML;
   }
 }
 
-
+using namespace VSFileSystem;
 void StarSystem::LoadXML(const char *filename, const Vector & centroid, const float timeofyear) {
 using namespace StarXML;
   //  shield.number=0;
-  const int chunk_size = 16384;
+  // const int chunk_size = 16384;
   // rrestricted=yrestricted=prestricted=false;
   bool autogenerated=false;
   this->filename = filename;
-  string file = GetCorrectStarSysPath (filename,autogenerated);
+  string file = VSFileSystem::GetCorrectStarSysPath (filename,autogenerated);
   static bool game_speed_affects_autogen_systems=XMLSupport::parse_bool(vs_config->getVariable ("physics","game_speed_affects_autogen_systems","false"));
   if (game_speed_affects_autogen_systems) {
     autogenerated = false;
   }
-  FILE * inFile=NULL;
-  if (file.length()) {
-    inFile = fopen (file.c_str(), "r");
-    if(!inFile) {
+  VSFile f;
+  VSError err;
+  //if (file.length()) {
+	err = f.OpenReadOnly( filename, SystemFile);
+    if(err>Ok) {
       printf("StarSystem: file not found %s\n",filename);
       
       return;
     }
+  /*
   }else {
     printf("StarSystem: file not found %s\n",filename);
     return;
   }
+  */
 
   xml = new StarXML;
   xml->scale=1;
   xml->fade = autogenerated;
-  if (!autogenerated) {
-    xml->scale*=parse_float (vs_config->getVariable("physics","game_speed","1"));
-  }
+  //if (!autogenerated) {  // Originally implemented to keep travel time constant in hand tuned systems.  However, it really tweaks out other things, so we've turned it off.
+  //  xml->scale*=parse_float (vs_config->getVariable("physics","game_speed","1"));
+  //}
   xml->scale*=parse_float (vs_config->getVariable("physics","star_system_scale","1"));
 		
 
@@ -1284,9 +1277,9 @@ using namespace StarXML;
   xml->systemcentroid=centroid;
   xml->timeofyear = timeofyear;
   //xml->fade is only if autogenerated
-  xml->starsp = GetStarSpreadScale(150);
-  xml->numnearstars=GetNumNearStarsScale(400);
-  xml->numstars=GetNumStarsScale(800);
+  xml->starsp = GetStarSpreadScale();
+  xml->numnearstars=GetNumNearStarsScale();
+  xml->numstars=GetNumStarsScale();
   xml->backgroundname = string("cube");
   xml->reflectivity=XMLSupport::parse_float (vs_config->getVariable("graphics","reflectivity",".2"));
   xml->unitlevel=0;
@@ -1294,15 +1287,18 @@ using namespace StarXML;
   XML_SetUserData(parser, this);
   XML_SetElementHandler(parser, &StarSystem::beginElement, &StarSystem::endElement);
   
+  XML_Parse(parser, (f.ReadFull()).c_str(), f.Size(), 1);
+ /*
   do {
     char *buf = (XML_Char*)XML_GetBuffer(parser, chunk_size);
     int length;
     
-    length = fread (buf,1, chunk_size,inFile);
+    length = VSFileSystem::vs_read (buf,1, chunk_size,inFile);
     //length = inFile.gcount();
-    XML_ParseBuffer(parser, length, feof(inFile));
-  } while(!feof(inFile));
-  fclose (inFile);
+    XML_ParseBuffer(parser, length, VSFileSystem::vs_feof(inFile));
+  } while(!VSFileSystem::vs_feof(inFile));
+  */
+  f.Close();
   XML_ParserFree (parser);
   UnIterator * iter;
   unsigned int i;
@@ -1312,6 +1308,7 @@ using namespace StarXML;
       Unit * un;
       while ((un = iter->current())) {
 	drawList.prepend(un);
+        units.prepend(un);
 	iter->advance();
       }
       delete iter;
